@@ -40,10 +40,12 @@ struct IRCConf
   int port;
   int tls;
   int tls_hostname_verification;
+  char *rsa_private_key_file;
+  char *tls_certificate_file;
   time_t readtimeout;
   time_t reconnectinterval;
   char *password;
-  char *vhost;
+  char *bind;
   char *nickserv;
   char *oper;
   char *mode;
@@ -88,7 +90,7 @@ struct ScannerConf
   node_t node;  /**< List node; linked into ScannerItemList */
   char *name;
   list_t protocols;
-  char *vhost;
+  char *bind;
   int fd;
   char *target_ip;
   int target_port;
